@@ -20,7 +20,7 @@ our %EXPORT_TAGS = (
     ':override' => 'internal',
     );
 
-our $VERSION = '1.30_01';
+our $VERSION = '1.31_01';
 
 bootstrap Time::Piece $VERSION;
 
@@ -31,6 +31,7 @@ my @FULLMON_LIST = qw(January February March April May June July
                       August September October November December);
 my @DAY_LIST = qw(Sun Mon Tue Wed Thu Fri Sat);
 my @FULLDAY_LIST = qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
+my $IS_WIN32 = ($^O =~ /Win32/);
 
 use constant {
     'c_sec' => 0,
@@ -456,20 +457,88 @@ sub month_last_day {
     return $MON_LAST[$_mon] + ($_mon == 1 ? _is_leap_year($year) : 0);
 }
 
-#since %z and %Z are not portable lets just
-#parse it out before calling native strftime
-#(but only if we are in UTC time)
-my %GMT_REPR = (
-    '%z' => '+0000',
-    '%Z' => 'UTC',
-);
+my $strftime_trans_map = {
+    'c' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%c/%a %d %b %Y %I:%M:%S %p/;
+        return $format;
+    },
+    'e' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%e/%d/ if $IS_WIN32;
+        return $format;
+    },
+    'D' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%D/%m\/%d\/%y/;
+        return $format;
+    },
+    'F' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%F/%Y-%m-%d/;
+        return $format;
+    },
+    'r' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%r/%I:%M:%S %p/ if $IS_WIN32;
+        return $format;
+    },
+    'R' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%R/%H:%M/;
+        return $format;
+    },
+    's' => sub {
+        #%s not portable if time parts are from gmtime since %s will
+        #cause a call to native mktime (and thus uses local TZ)
+        my ( $time, $format ) = @_;
+        $format =~ s/%s/$time->[c_epoch]/;
+        return $format;
+    },
+    'T' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%T/%H:%M:%S/ if $IS_WIN32;
+        return $format;
+    },
+    'u' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%u/%w/ if $IS_WIN32;
+        return $format;
+    },
+    'V' => sub {
+        my ( $time, $format ) = @_;
+        my $week = sprintf( "%02d", $time->week() );
+        $format =~ s/%V/$week/ if $IS_WIN32;
+        return $format;
+    },
+    'x' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%x/%a %d %b %Y/;
+        return $format;
+    },
+    'X' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%X/%I:%M:%S %p/;
+        return $format;
+    },
+    'z' => sub {    #%[zZ] not portable if time parts are from gmtime
+        my ( $time, $format ) = @_;
+        $format =~ s/%z/+0000/ if not $time->[c_islocal];
+        return $format;
+    },
+    'Z' => sub {
+        my ( $time, $format ) = @_;
+        $format =~ s/%Z/UTC/ if not $time->[c_islocal];
+        return $format;
+    },
+};
 
 sub strftime {
     my $time = shift;
     my $format = @_ ? shift(@_) : '%a, %d %b %Y %H:%M:%S %Z';
-    if (! $time->[c_islocal]) {
-        $format =~ s/(%.)/$GMT_REPR{$1} || $1/eg;
-    }
+    $format = $time->_translate_format($format, $strftime_trans_map);
+
+    return $format unless $format =~ /%/; #if translate removes everything
 
     return _strftime($format, $time->epoch, $time->[c_islocal]);
 }
@@ -478,9 +547,10 @@ sub strptime {
     my $time = shift;
     my $string = shift;
     my $format = @_ ? shift(@_) : "%a, %d %b %Y %H:%M:%S %Z";
-    my @vals = _strptime($string, $format);
-#    warn(sprintf("got vals: %d-%d-%d %d:%d:%d\n", reverse(@vals)));
-    return scalar $time->_mktime(\@vals, (ref($time) ? $time->[c_islocal] : 0));
+    my $islocal = (ref($time) ? $time->[c_islocal] : 0);
+    my @vals = _strptime($string, $format, $islocal);
+#    warn(sprintf("got vals: %d-%d-%d %d:%d:%d\n", reverse(@vals[c_sec..c_year])));
+    return scalar $time->_mktime(\@vals, $islocal);
 }
 
 sub day_list {
@@ -634,6 +704,62 @@ sub add_years {
     $time->add_months($years * 12);
 }
 
+sub truncate {
+    my ($time, %params) = @_;
+    return $time unless exists $params{to};
+    #if ($params{to} eq 'week') { return $time->_truncate_week; }
+    my %units = (
+        second   => 0,
+        minute   => 1,
+        hour     => 2,
+        day      => 3,
+        month    => 4,
+        quarter  => 5,
+        year     => 5
+    );
+    my $to = $units{$params{to}};
+    croak "Invalid value of 'to' parameter: $params{to}" unless defined $to;
+    my $start_month = 0;
+    if ($params{to} eq 'quarter') {
+        $start_month = int( $time->_mon / 3 ) * 3;
+    }
+    my @down_to = (0, 0, 0, 1, $start_month, $time->year);
+    return $time->_mktime([@down_to[0..$to-1], @$time[$to..c_isdst]],
+        $time->[c_islocal]);
+}
+
+#Given a format and a translate map, replace format flags in
+#accordance with the logic from the translation map subroutines
+sub _translate_format {
+    my ( $time, $format, $trans_map ) = @_;
+
+    $format =~ s/%%/\e\e/g; #escape the escape
+    my $lexer = _build_format_lexer($format);
+
+	while(my $flag = $lexer->() ){
+        next unless exists $trans_map->{$flag};
+		$format = $trans_map->{$flag}($time, $format);
+	}
+
+    $format =~ s/\e\e/%%/g;
+    return $format;
+}
+
+sub _build_format_lexer {
+    my $format = shift();
+
+    #Higher Order Perl p.359 (or thereabouts)
+    return sub {
+        LABEL: {
+        return $1 if $format =~ m/\G%([a-zA-Z])/gc; #return single char flags
+
+        redo LABEL if $format =~ m/\G(.)/gc;
+        return; #return at empty string
+        }
+    };
+}
+
+
 1;
 __END__
 
@@ -658,7 +784,7 @@ in perlfunc will still return what you expect.
 
 The module actually implements most of an interface described by
 Larry Wall on the perl5-porters mailing list here:
-http://www.xray.mpe.mpg.de/mailing-lists/perl5-porters/2000-01/msg00241.html
+L<http://www.xray.mpe.mpg.de/mailing-lists/perl5-porters/2000-01/msg00241.html>
 
 =head1 USAGE
 
@@ -783,8 +909,8 @@ days, weeks and years in that delta, using the Time::Seconds API.
 In addition to adding seconds, there are two APIs for adding months and
 years:
 
-    $t->add_months(6);
-    $t->add_years(5);
+    $t = $t->add_months(6);
+    $t = $t->add_years(5);
 
 The months and years can be negative for subtractions. Note that there
 is some "strange" behaviour when adding and subtracting months at the
@@ -794,6 +920,17 @@ subtracting a month from 2008-03-31 will not result in 2008-02-31 as this
 is an impossible date. Instead you will get 2008-03-02. This appears to
 be consistent with other date manipulation tools.
 
+=head2 Truncation
+
+Calling the C<truncate> method returns a copy of the object but with the
+time truncated to the start of the supplied unit.
+
+    $t = $t->truncate(to => 'day');
+
+This example will set the time to midnight on the same date which C<$t>
+had previously. Allowed values for the "to" parameter are: "year",
+"quarter", "month", "day", "hour", "minute" and "second".
+
 =head2 Date Comparisons
 
 Date comparisons are also possible, using the full suite of "<", ">",
@@ -818,7 +955,7 @@ Outputs:
 For more information see "man strptime", which should be on all unix
 systems.
 
-Alternatively look here: http://www.unix.com/man-page/FreeBSD/3/strftime/
+Alternatively look here: L<http://www.unix.com/man-page/FreeBSD/3/strftime/>
 
 =head2 YYYY-MM-DDThh:mm:ss
 
@@ -886,7 +1023,7 @@ as Perl.
 
 =head1 SEE ALSO
 
-The excellent Calendar FAQ at http://www.tondering.dk/claus/calendar.html
+The excellent Calendar FAQ at L<http://www.tondering.dk/claus/calendar.html>
 
 =head1 BUGS
 
